@@ -16,6 +16,8 @@
     with this program; if not, write to the Free Software Foundation, Inc.,
     51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */
+#include "connectdialog.h"
+#include "fullscreenexitbutton.h"
 #include "mainwindow.h"
 #include "preferences.h"
 #include "vncview.h"
@@ -31,7 +33,7 @@
 MainWindow::MainWindow(QString url, int quality):
 	QMainWindow(0),
 	vnc_view(0),
-	scroll_area(new QScrollArea(0))
+	scroll_area(new ScrollArea(0))
 {
 	setWindowTitle("Presence VNC");
 	setAttribute(Qt::WA_Maemo5StackedWindow);
@@ -42,45 +44,48 @@ MainWindow::MainWindow(QString url, int quality):
 
 	//set up toolbar
 	toolbar = new QToolBar(0);
-	//toolbar->setAttribute(Qt::WA_InputMethodEnabled, true);
 	toolbar->addAction("Mod", this, SLOT(showModifierMenu()));
 	toolbar->addAction("Tab", this, SLOT(sendTab()));
 	toolbar->addAction("Esc", this, SLOT(sendEsc()));
 	toolbar->addAction("PgUp", this, SLOT(sendPgUp()));
 	toolbar->addAction("PgDn", this, SLOT(sendPgDn()));
-//	toolbar->addAction("IM", toolbar, SLOT(setFocus())); //doesn't work
+	toolbar->addAction(QIcon("/usr/share/icons/hicolor/48x48/hildon/chat_enter.png"), "", this, SLOT(sendReturn()));
+	toolbar->addAction(QIcon("/usr/share/icons/hicolor/48x48/hildon/control_keyboard.png"), "", this, SLOT(showInputPanel()));
+
+	//move fullscreen button to the right
+	QWidget *spacer = new QWidget();
+	spacer->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Expanding);
+	toolbar->addWidget(spacer);
+
 	toolbar->addAction(QIcon("/usr/share/icons/hicolor/48x48/hildon/general_fullsize.png"), "", this, SLOT(toggleFullscreen()));
 	addToolBar(toolbar);
 	toolbar->setVisible(settings.value("show_toolbar", true).toBool());
 
 	//set up menu
 	QMenuBar *menu = new QMenuBar(this);
-	QAction *connect_action = new QAction("Connect", this);
-	disconnect_action = new QAction("Disconnect", this);
+	QAction *connect_action = new QAction(tr("Connect"), this);
+	disconnect_action = new QAction(tr("Disconnect"), this);
 	menu->addAction(connect_action);
 	menu->addAction(disconnect_action);
-	scaling = new QAction("Fit to Screen", this);
+	scaling = new QAction(tr("Fit to Screen"), this);
 	scaling->setCheckable(true);
 	scaling->setChecked(settings.value("rescale", true).toBool());
 	menu->addAction(scaling);
-	QAction *show_toolbar = new QAction("Show Toolbar", this);
+	show_toolbar = new QAction(tr("Show Toolbar"), this);
 	show_toolbar->setCheckable(true);
 	show_toolbar->setChecked(settings.value("show_toolbar", true).toBool());
 	menu->addAction(show_toolbar);
-	QAction *pref_action = new QAction("Preferences", this);
+	QAction *pref_action = new QAction(tr("Preferences"), this);
 	menu->addAction(pref_action);
-	QAction *about_action = new QAction("About", this);
+	QAction *about_action = new QAction(tr("About"), this);
 	menu->addAction(about_action);
 
-	//menu->setAttribute(Qt::WA_Maemo5StackedWindow);
-	//menu->hide();
-
 	connect(about_action, SIGNAL(triggered()),
 		this, SLOT(about()));
 	connect(pref_action, SIGNAL(triggered()),
 		this, SLOT(showPreferences()));
 	connect(connect_action, SIGNAL(triggered()),
-		this, SLOT(connectDialog()));
+		this, SLOT(showConnectDialog()));
 	connect(disconnect_action, SIGNAL(triggered()),
 		this, SLOT(disconnectFromHost()));
 	connect(show_toolbar, SIGNAL(toggled(bool)),
@@ -89,6 +94,7 @@ MainWindow::MainWindow(QString url, int quality):
 		this, SLOT(forceResizeDelayed()));
 
 	setCentralWidget(scroll_area);
+	new FullScreenExitButton(this);
 
 	grabZoomKeys(true);
 	reloadSettings();
@@ -99,7 +105,7 @@ MainWindow::MainWindow(QString url, int quality):
 	if(url.isNull()) {
 		disconnect_action->setEnabled(false);
 		toolbar->setEnabled(false);
-		connectDialog();
+		showConnectDialog();
 	} else {
 		vnc_view = new VncView(this, url, RemoteView::Quality(quality));
 		connect(scaling, SIGNAL(toggled(bool)),
@@ -128,7 +134,7 @@ void MainWindow::closeEvent(QCloseEvent*) {
 	grabZoomKeys(false);
 
 	QSettings settings;
-	settings.setValue("show_toolbar", toolbar->isVisible());
+	settings.setValue("show_toolbar", show_toolbar->isChecked());
 	settings.setValue("rescale", scaling->isChecked());
 	settings.sync();
 
@@ -139,7 +145,7 @@ void MainWindow::closeEvent(QCloseEvent*) {
 
 void MainWindow::about() {
 	QMessageBox::about(this, tr("About Presence VNC"),
-		tr("<center><h1>Presence VNC 0.3</h1>\
+		tr("<center><h1>Presence VNC 0.4</h1>\
 A touchscreen friendly VNC client\
 <small><p>&copy;2010 Christian Pulvermacher &lt;pulvermacher@gmx.de&gt;</p>\
 <p>Based on KRDC, &copy; 2007-2008 Urs Wolfer</p>\
@@ -147,16 +153,16 @@ A touchscreen friendly VNC client\
 <p>This program is free software; License: <a href=\"http://www.gnu.org/licenses/gpl-2.0.html\">GNU GPL 2</a> or later.</p>"));
 }
 
-void MainWindow::connectDialog()
+void MainWindow::showConnectDialog()
 {
-	QSettings settings;
-	QString url = QInputDialog::getText(this, "Connect to Host", "VNC Server:", QLineEdit::Normal, settings.value("last_hostname", "").toString());
-	if(url.isEmpty()) { //dialog dismissed or nothing entered
-		return;
-	}
-	settings.setValue("last_hostname", url);
-	url = "vnc://" + url;
+	ConnectDialog *connect_dialog = new ConnectDialog(this);
+	connect(connect_dialog, SIGNAL(connectToHost(QString)),
+		this, SLOT(connectToHost(QString)));
+	connect_dialog->exec();
+}
 
+void MainWindow::connectToHost(QString url)
+{
 	disconnectFromHost();
 
 	vnc_view = new VncView(this, url, RemoteView::Quality(2)); //TODO: get quality in dialog
@@ -205,7 +211,7 @@ void MainWindow::statusChanged(RemoteView::RemoteStatus status)
 		break;
 	case RemoteView::Disconnecting:
 		if(old_status != RemoteView::Disconnected) { //Disconnecting also occurs while connecting, so check last state
-			QMaemo5InformationBox::information(this, "Connection lost");
+			QMaemo5InformationBox::information(this, tr("Connection lost"));
 			
 			//clean up
 			scroll_area->setWidget(0);
@@ -245,6 +251,7 @@ void MainWindow::forceResizeDelayed()
 
 void MainWindow::toggleFullscreen()
 {
+	toolbar->setVisible(show_toolbar->isChecked() and (windowState() & Qt::WindowFullScreen)); //hide toolbar in fullscreen
 	setWindowState(windowState() ^ Qt::WindowFullScreen); 
 	forceResizeDelayed();
 }
@@ -290,3 +297,10 @@ void MainWindow::reloadSettings()
 	if(vnc_view)
 		vnc_view->reloadSettings();
 }
+
+void MainWindow::showInputPanel()
+{
+	vnc_view->setAttribute(Qt::WA_InputMethodEnabled, true);
+	QEvent event(QEvent::RequestSoftwareInputPanel);
+	QApplication::sendEvent(vnc_view, &event);
+}
