@@ -28,7 +28,7 @@ void migrateConfiguration()
 {
 	QSettings settings;
 	int config_ver = settings.value("config_version", 0).toInt();
-	if(config_ver == 1) //config file up-to-date
+	if(config_ver == 2) //config file up-to-date
 		return;
 
 	std::cout << "Migrating from configuration ver. " << config_ver << "\n";
@@ -42,6 +42,20 @@ void migrateConfiguration()
 			settings.setValue("left_zoom", right_zoom+1);
 		config_ver = 1;
 	}
+	if(config_ver == 1) {
+		QString last_hostname = settings.value("last_hostname", "").toString();
+		settings.remove("last_hostname");
+		if(!last_hostname.isEmpty()) {
+			//make sure hostname is sane
+			last_hostname.remove(QChar('/'));
+			last_hostname.remove(QChar('\\'));
+			last_hostname = last_hostname.toLower();
+
+			settings.setValue(QString("hosts/%1/position").arg(last_hostname), 0);
+		}
+		
+		config_ver = 2;
+	}
 	settings.setValue("config_version", config_ver);
 	settings.sync();
 }
@@ -50,7 +64,7 @@ void migrateConfiguration()
 Preferences::Preferences(QWidget *parent):
 	QDialog(parent)
 {
-	setWindowTitle("Preferences");
+	setWindowTitle(tr("Preferences"));
 
 	QHBoxLayout *layout1 = new QHBoxLayout();
 	QVBoxLayout *layout2 = new QVBoxLayout();
@@ -95,7 +109,7 @@ Preferences::Preferences(QWidget *parent):
 	disable_tapping->setChecked(settings.value("disable_tapping", false).toBool());
 	layout2->addWidget(disable_tapping);
 
-	QPushButton *ok = new QPushButton("OK");
+	QPushButton *ok = new QPushButton(tr("Done"));
 	ok->setMaximumWidth(100);
 
 	layout1->addLayout(layout2);
