@@ -30,14 +30,20 @@
 critical(parent, caption, message)
 
 #include <QApplication>
+#include <QCheckBox>
+#include <QDialog>
 #include <QImage>
+#include <QHBoxLayout>
+#include <QVBoxLayout>
 #include <QPainter>
 #include <QMouseEvent>
+#include <QPushButton>
 #include <QEvent>
 #include <QSettings>
 #include <QTime>
 #include <QTimer>
 
+
 // Definition of key modifier mask constants
 #define KMOD_Alt_R 	0x01
 #define KMOD_Alt_L 	0x02
@@ -89,7 +95,6 @@ VncView::~VncView()
 
 void VncView::forceFullRepaint()
 {
-	kDebug(5011) << "forceFullRepaint()";
 	force_full_repaint = true;
 	repaint();
 }
@@ -213,17 +218,54 @@ void VncView::requestPassword()
         return;
     }
 
-    bool ok;
-    QString password = QInputDialog::getText(this, //krazy:exclude=qclasses
-                                             tr("Password required"),
-                                             tr("Please enter the password for the remote desktop:"),
-                                             QLineEdit::Password, QString(), &ok);
-    m_firstPasswordTry = false;
-    if (ok) {
-        vncThread.setPassword(password);
-    } else {
-        startQuitting();
-    }
+	QSettings settings;
+	settings.beginGroup("hosts");
+	QString password = settings.value(QString("%1/password").arg(m_host), "").toString();
+	//check for saved password
+	if(m_firstPasswordTry and !password.isEmpty()) {
+		kDebug(5011) << "Trying saved password";
+		m_firstPasswordTry = false;
+		vncThread.setPassword(password);
+		return;
+	}
+	m_firstPasswordTry = false;
+
+	//build dialog
+	QDialog dialog(this);
+	dialog.setWindowTitle(tr("Password required"));
+
+	QLineEdit passwordbox;
+	passwordbox.setEchoMode(QLineEdit::Password);
+	passwordbox.setText(password);
+	QCheckBox save_password(tr("Save Password"));
+	save_password.setChecked(!password.isEmpty()); //offer to overwrite saved password
+	QPushButton ok_button(tr("Done"));
+	ok_button.setMaximumWidth(100);
+	connect(&ok_button, SIGNAL(clicked()),
+		&dialog, SLOT(accept()));
+
+	QHBoxLayout layout1;
+	QVBoxLayout layout2;
+	layout2.addWidget(&passwordbox);
+	layout2.addWidget(&save_password);
+	layout1.addLayout(&layout2);
+	layout1.addWidget(&ok_button);
+	dialog.setLayout(&layout1);
+
+	if(dialog.exec()) { //dialog accepted
+		password = passwordbox.text();
+
+		if(save_password.isChecked()) {
+			kDebug(5011) << "Saving password for host '" << m_host << "'";
+
+			settings.setValue(QString("%1/password").arg(m_host), password);
+			settings.sync();
+		}
+
+		vncThread.setPassword(password);
+	} else {
+		startQuitting();
+	}
 }
 
 void VncView::outputErrorMessage(const QString &message)
@@ -243,7 +285,10 @@ void VncView::outputErrorMessage(const QString &message)
 
 void VncView::updateImage(int x, int y, int w, int h)
 {
-//     kDebug(5011) << "got update" << width() << height();
+	if(!QApplication::focusWidget()) { //no focus, we're probably minimized
+		return;
+	}
+     //kDebug(5011) << "got update" << width() << height();
 
     m_x = x;
     m_y = y;
@@ -252,10 +297,13 @@ void VncView::updateImage(int x, int y, int w, int h)
 
     if (m_horizontalFactor != 1.0 || m_verticalFactor != 1.0) {
         // If the view is scaled, grow the update rectangle to avoid artifacts
-        m_x-=1;
-        m_y-=1;
-        m_w+=2;
-        m_h+=2;
+        int x_extrapixels = 1.0/m_horizontalFactor+1;
+        int y_extrapixels = 1.0/m_verticalFactor+1;
+
+        m_x-=x_extrapixels;
+        m_y-=y_extrapixels;
+        m_w+=2*x_extrapixels;
+        m_h+=2*y_extrapixels;
     }
 
     m_frame = vncThread.image();
@@ -375,10 +423,10 @@ void VncView::paintEvent(QPaintEvent *event)
                                                                   qRound(m_h*m_verticalFactor),
                                                                   Qt::IgnoreAspectRatio, Qt::SmoothTransformation));
     } else {
-         kDebug(5011) << "resize repaint";
+         //kDebug(5011) << "resize repaint";
         QRect rect = event->rect();
         if (!force_full_repaint and (rect.width() != width() || rect.height() != height())) {
-             kDebug(5011) << "Partial repaint";
+          //   kDebug(5011) << "Partial repaint";
             const int sx = rect.x()/m_horizontalFactor;
             const int sy = rect.y()/m_verticalFactor;
             const int sw = rect.width()/m_horizontalFactor;
@@ -427,6 +475,9 @@ bool VncView::event(QEvent *event)
         wheelEventHandler(static_cast<QWheelEvent*>(event));
         return true;
         break;
+    case QEvent::WindowActivate: //input panel may have been closed, prevent IM from interfering with hardware keyboard
+	setAttribute(Qt::WA_InputMethodEnabled, false);
+	//fall through
     default:
         return RemoteView::event(event);
     }
@@ -441,7 +492,7 @@ void VncView::mouseEventHandler(QMouseEvent *e)
 	static QTime press_time;
 	static QTime up_time; //used for double clicks/tap&drag, for time after first tap
 
-	const int TAP_PRESS_TIME = 200;
+	const int TAP_PRESS_TIME = 180;
 	const int DOUBLE_TAP_UP_TIME = 500;
 
 	if(!e) { //flush held taps
@@ -545,8 +596,9 @@ void VncView::wheelEventHandler(QWheelEvent *event)
 void VncView::keyEventHandler(QKeyEvent *e)
 {
     // strip away autorepeating KeyRelease; see bug #206598
-    if (e->isAutoRepeat() && (e->type() == QEvent::KeyRelease))
+    if (e->isAutoRepeat() && (e->type() == QEvent::KeyRelease)) {
         return;
+    }
 
 // parts of this code are based on http://italc.sourcearchive.com/documentation/1.0.9.1/vncview_8cpp-source.html
     rfbKeySym k = e->nativeVirtualKey();
@@ -576,8 +628,13 @@ void VncView::keyEventHandler(QKeyEvent *e)
 		current_zoom = left_zoom;
 	else if(e->key() == Qt::Key_F7)
 		current_zoom = right_zoom;
-	else if (k)
+	else if (k) {
+	//	kDebug(5011) << "got '" << e->text() << "'.";
 		vncThread.keyEvent(k, pressed);
+	} else {
+		kDebug(5011) << "nativeVirtualKey() for '" << e->text() << "' failed.";
+		return;
+	}	
 	
 	if(current_zoom == -1)
 		return;
@@ -674,6 +731,9 @@ void VncView::sendKey(Qt::Key key)
 	case Qt::Key_PageDown:
 		k = 0xff56;
 		break;
+	case Qt::Key_Return:
+		k = 0xff0d;
+		break;
 	case Qt::Key_Meta: //TODO: test this.
 		k = XK_Super_L;
 		break;
@@ -681,7 +741,7 @@ void VncView::sendKey(Qt::Key key)
 		k = XK_Alt_L;
 		break;
 	default:
-		kDebug(5011) << "unhandled Qt::Key value " << key;
+		kDebug(5011) << "sendKey(): Unhandled Qt::Key value " << key;
 		return;
 	}
 
@@ -707,4 +767,24 @@ void VncView::reloadSettings()
 	disable_tapping = settings.value("disable_tapping", false).toBool();
 }
 
+//convert commitString into keyevents
+void VncView::inputMethodEvent(QInputMethodEvent *event)
+{
+	//TODO handle replacements
+	//NOTE for the return key to work Qt needs to enable multiline input, which only works for Q(Plain)TextEdit
+
+	//kDebug(5011) << event->commitString() << "|" << event->preeditString() << "|" << event->replacementLength() << "|" << event->replacementStart();
+	QString letters = event->commitString();
+	for(int i = 0; i < letters.length(); i++) {
+		char k = letters.at(i).toLatin1(); //works with all 'normal' keys, not umlauts.
+		if(!k) {
+			kDebug(5011) << "unhandled key";
+			continue;
+		}
+		vncThread.keyEvent(k, true);
+		vncThread.keyEvent(k, false);
+	}
+}
+
+
 #include "moc_vncview.cpp"
