@@ -30,6 +30,7 @@
 class KConfigGroup{};
 
 #include <QClipboard>
+#include <QTextEdit>
 
 extern "C" {
 #include <rfb/rfbclient.h>
@@ -68,6 +69,7 @@ protected:
     bool event(QEvent *event);
     void resizeEvent(QResizeEvent *event);
     bool eventFilter(QObject *obj, QEvent *event);
+    void inputMethodEvent(QInputMethodEvent *event);
 
 private:
     VncClientThread vncThread;
